<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    
    // Iniciar transacción
    //mysqli_begin_transaction($conn);
    $nombre_tabla = 'caja';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
    if (!isset($datos['sucursal_id'], $datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['sucursal_id']) || empty($datos['token']) || trim($datos['sucursal_id']) === '' || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array
    $token = $datos['token'];
    $sucursal_id = isset($datos['sucursal_id']) ? mysqli_real_escape_string($conn, $datos['sucursal_id']) : '';
    $usuario_id = isset($datos['usuario_id']) ? mysqli_real_escape_string($conn, $datos['usuario_id']) : '';
    $estado_apertura = isset($datos['estado_apertura']) ? mysqli_real_escape_string($conn, $datos['estado_apertura']) : '';
    $monto_inicial = isset($datos['monto_inicial']) ? mysqli_real_escape_string($conn, $datos['monto_inicial']) : '';
    $fecha_apertura = isset($datos['fecha_apertura']) ? mysqli_real_escape_string($conn, $datos['fecha_apertura']) : '';
    $hora_apertura = isset($datos['hora_apertura']) ? mysqli_real_escape_string($conn, $datos['hora_apertura']) : '';
    $estado_cierre = isset($datos['estado_cierre']) ? mysqli_real_escape_string($conn, $datos['estado_cierre']) : '';
    $fecha_cierre = isset($datos['fecha_cierre']) ? mysqli_real_escape_string($conn, $datos['fecha_cierre']) : '';
    $hora_cierre = isset($datos['hora_cierre']) ? mysqli_real_escape_string($conn, $datos['hora_cierre']) : '';
    $monto_cierre = isset($datos['monto_cierre']) ? mysqli_real_escape_string($conn, $datos['monto_cierre']) : '';

    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para aperturar caja", 403);
    }

    // Construir y ejecutar la consulta SQL
    $sql_insert_sucursal = "INSERT INTO caja(sucursal_id, usuario_id, estado_apertura,monto_inicial,fecha_apertura,hora_apertura,estado_cierre,fecha_cierre,hora_cierre,monto_cierre) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    $stmt = mysqli_prepare($conn, $sql_insert_sucursal);
    mysqli_stmt_bind_param($stmt, "ssssssssss", $sucursal_id, $usuario_id, $estado_apertura, $monto_inicial, $fecha_apertura, $hora_apertura, $estado_cierre, $fecha_cierre, $hora_cierre, $monto_cierre);
    if (!mysqli_stmt_execute($stmt)) {
        throw new Exception("Error al guardar los datos de la sucursal: " . mysqli_error($conn), 500);
    }
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    //mysqli_commit($conn);

    $resultado['ESTADO'] = "1";
    $resultado['MENSAJE'] = "La informacion se registro correctamente";
    $resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
   // mysqli_rollback($conn);
    
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
