<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    
    // Iniciar transaccion
    //mysqli_begin_transaction($conn);
    
    $nombre_tabla = 'caja';
    $tipo_permiso = 'listar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
     if (!isset($datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no este vacio y no consiste solo en espacios en blanco
    if (empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacios o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array
    $token = isset($datos['token']) ? $datos['token'] : null;
    $ID_Sucursal = isset($datos['sucursal_id']) ? $datos['sucursal_id'] : null;
    $Fecha_inicio = isset($datos['Fecha_inicio']) ? $datos['Fecha_inicio'] : null;
    $Fecha_fin = isset($datos['Fecha_fin']) ? $datos['Fecha_fin'] : null;
    $ID_Usuario = isset($datos['usuario_id']) ? $datos['usuario_id'] : null;

    // Verificar si no se enviaron valores para los campos de fecha
    if (empty($Fecha_inicio) && empty($Fecha_fin)) {
        // Obtener el rango de fechas del presente mes
        $Fecha_inicio = date('Y-m-01');
        $Fecha_fin = date('Y-m-t');
    }

    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para editar en esta tabla.", 403);
    }

    // Construir y ejecutar la consulta SQL
$sql = "SELECT  MC.*,  U.id, U.correo,  S.ID_Sucursal,S.Nombre_Sucursal, E.Nombre_Completo AS nombrempleado
    FROM
    Movimientos_Caja MC
    INNER JOIN usuarios U ON MC.ID_Usuario = U.id
    INNER JOIN Sucursales S ON MC.ID_Sucursal = S.ID_Sucursal
    INNER JOIN Empleados E ON U.ID_Empleado = E.ID_Empleado
        WHERE 1 = 1";

// Verificar las combinaciones de parámetros
$params = array();
$types = '';

if (!empty($ID_Sucursal)) {
    $sql .= " AND MC.ID_Sucursal = ?";
    $params[] = $ID_Sucursal;
    $types .= 'i';
}

if (!empty($ID_Usuario)) {
    $sql .= " AND MC.ID_Usuario = ?";
    $params[] = $ID_Usuario;
    $types .= 'i';
}

if (!empty($Fecha_inicio) && !empty($Fecha_fin)) {
    $sql .= " AND MC.Fecha BETWEEN ? AND ?";
    $params[] = $Fecha_inicio;
    $params[] = $Fecha_fin;
    $types .= 'ss';
}

$sql .= " ORDER BY MC.Fecha DESC, MC.Hora ASC";

// Preparar la consulta
$stmt = mysqli_prepare($conn, $sql);
if ($stmt) {
    // Agregar tipos de parámetros
    if (!empty($types)) {
        mysqli_stmt_bind_param($stmt, $types, ...$params);
    }

    // Ejecutar la consulta
    if (mysqli_stmt_execute($stmt)) {
        // Obtener resultados
        $result = mysqli_stmt_get_result($stmt);
        $rows = mysqli_fetch_all($result, MYSQLI_ASSOC);
        mysqli_free_result($result);

        // Comprobar si hay datos
        if (count($rows) > 0) {
            print_json($rows, 1, "Registros disponibles.", 200);
        } else {
            print_json([], 0, "No se encontraron registros.", 400);
        }
    } else {
        throw new Exception("ERROR: Could not able to execute query. " . mysqli_error($conn));
    }

    // Cerrar la declaracion
    mysqli_stmt_close($stmt);
} else {
    throw new Exception("ERROR: Could not prepare query. " . mysqli_error($conn));
}
    
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    //mysqli_commit($conn);

} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
    //mysqli_rollback($conn);

    print_json([], 0, $e->getMessage());
}

mysqli_close($conn);

function print_json($data, $estado = 1, $mensaje) {
    $response = array(
        "server_mensaje" => array(
            "ESTADO" => $estado,
            "MENSAJE" => $mensaje
        ),
        "data" => $data
    );

    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
