<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    
    // Iniciar transaccion
    //mysqli_begin_transaction($conn);
    
    $nombre_tabla = 'Cabecera_CreditoVentas';
    $tipo_permiso = 'editar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
    if (!isset($datos['token'], $datos['ID_Cab_CreditoV'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no esten vacio y no consiste solo en espacios en blanco
    if (empty($datos['ID_Cab_CreditoV']) || empty($datos['token']) || trim($datos['ID_Cab_CreditoV']) === '' || trim($datos['token']) === '') {
        throw new Exception("Campos vacios o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array
     $token = $datos['token'];
     $ID_Cab_CreditoV = isset($datos['ID_Cab_CreditoV']) ? mysqli_real_escape_string($conn, $datos['ID_Cab_CreditoV']) : '';
     $Estado_Cancelado = isset($datos['Estado_Cancelado']) ? mysqli_real_escape_string($conn, $datos['Estado_Cancelado']) : '';
     $Acuenta = isset($datos['Acuenta']) ? mysqli_real_escape_string($conn, $datos['Acuenta']) : '';
     $Pendiente = isset($datos['Pendiente']) ? mysqli_real_escape_string($conn, $datos['Pendiente']) : '';
     $Total = isset($datos['Total']) ? mysqli_real_escape_string($conn, $datos['Total']) : '';
     

    // Verificar autorizacion del usuario y permisos 
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para editar esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para editar en esta tabla.", 403);
    }

    if($Estado_Cancelado=='1'){
       $Acuenta = '0';
       $Pendiente = '0'; 
    }

    // Construir y ejecutar la consulta SQL
    $sql_eliminar="UPDATE Cabecera_CreditoVentas SET
			Estado_Cancelado='$Estado_Cancelado',
			Acuenta='$Acuenta',
			Pendiente='$Pendiente'
            WHERE ID_Cab_CreditoV ='$ID_Cab_CreditoV'";
            

    if (!mysqli_query($conn, $sql_eliminar)) {
        throw new Exception("Error al editar Cabecera_CreditoVentas: " . mysqli_error($conn));
    }
    
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    //mysqli_commit($conn);

    $resultado['ESTADO'] = "1";
    $resultado['MENSAJE'] = "La informacion se actualizo correctamente";
    $resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
   // mysqli_rollback($conn);
    
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
