<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    // Iniciar transaccin
    mysqli_begin_transaction($conn);
    
    $nombre_tabla = 'Cabecera_Compras';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
     if (!isset($datos['ID_Sucursal'], $datos['ID_Proveedor'], $datos['Detalle'], $datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no est vaco y no consiste solo en espacios en blanco
    if (empty($datos['ID_Sucursal']) || empty($datos['token']) || empty($datos['ID_Proveedor']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacos o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array
    $token = $datos['token'];
    $ID_Sucursal = mysqli_real_escape_string($conn, $datos['ID_Sucursal']);
    $ID_Metodo_Pago = mysqli_real_escape_string($conn, $datos['ID_Metodo_Pago']);
    $ID_Proveedor = mysqli_real_escape_string($conn, $datos['ID_Proveedor']);
    $ID_Usuario = mysqli_real_escape_string($conn, $datos['ID_Usuario']);
    $ID_Moneda = mysqli_real_escape_string($conn, $datos['ID_Moneda']);//
    $Forma_Pago = mysqli_real_escape_string($conn, $datos['Forma_Pago']);//
    $NroComprobante = mysqli_real_escape_string($conn, $datos['NroComprobante']);
    $Estado_Activo = mysqli_real_escape_string($conn, $datos['Estado_Activo']);
    $Estado_Entrega = mysqli_real_escape_string($conn, $datos['Estado_Entrega']);
    $Estado_Canje = mysqli_real_escape_string($conn, $datos['Estado_Canje']);
    $Estado_Prestamo = mysqli_real_escape_string($conn, $datos['Estado_Prestamo']);
    $HoraCompra = mysqli_real_escape_string($conn, $datos['HoraCompra']);
    $Fecha_Compra = mysqli_real_escape_string($conn, $datos['Fecha_Compra']);
    $Fecha_vence = mysqli_real_escape_string($conn, $datos['Fecha_vence']);
    $Total_Grabadas = mysqli_real_escape_string($conn, $datos['Total_Grabadas']);
    $Total_Exoneradas = mysqli_real_escape_string($conn, $datos['Total_Exoneradas']);
    $Total_Gratuitas = mysqli_real_escape_string($conn, $datos['Total_Gratuitas']);
    $Total_Descuento = mysqli_real_escape_string($conn, $datos['Total_Descuento']);
    $Sub_Total = mysqli_real_escape_string($conn, $datos['Sub_Total']);
    $Porcentaje_IGV = mysqli_real_escape_string($conn, $datos['Porcentaje_IGV']);
    $Total_IGV = mysqli_real_escape_string($conn, $datos['Total_IGV']);
    $Total_ISC = mysqli_real_escape_string($conn, $datos['Total_ISC']);
    $Total_ICBPER = mysqli_real_escape_string($conn, $datos['Total_ICBPER']);
    $Total_Impuestos = mysqli_real_escape_string($conn, $datos['Total_Impuestos']);
    $Total_Otrs_Ompuestos = mysqli_real_escape_string($conn, $datos['Total_Otrs_Ompuestos']);
    $Total = mysqli_real_escape_string($conn, $datos['Total']);
    $Rucproveedor = mysqli_real_escape_string($conn, $datos['Rucproveedor']);
    $NombreProveedor = mysqli_real_escape_string($conn, $datos['NombreProveedor']);
    $DireccionProveedor = mysqli_real_escape_string($conn, $datos['DireccionProveedor']);
    $TotalLetras = mysqli_real_escape_string($conn, $datos['TotalLetras']);
    $restastock = mysqli_real_escape_string($conn, $datos['restastock']);
    $observacion = mysqli_real_escape_string($conn, $datos['observacion']);
    $Cod_Tipo_Documento = mysqli_real_escape_string($conn, $datos['Cod_Tipo_Documento']);

  $Acuenta= mysqli_real_escape_string($conn, $datos['Acuenta']);
  $Pendiente= mysqli_real_escape_string($conn, $datos['Pendiente']);
    
    
    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para editar en esta tabla.", 403); 
    }


// Insertar la cabecera de la compra
    $sql_insert_cabecera = "INSERT INTO `Cabecera_Compras` (`ID_Sucursal`, `ID_Metodo_Pago`, `ID_Proveedor`, `ID_Usuario`, `ID_Moneda`, `Forma_Pago`, `NroComprobante`, `Estado_Activo`, `Estado_Entrega`, `Estado_Canje`, `Estado_Prestamo`, `HoraCompra`, `Fecha_Compra`, `Total_Grabadas`, `Total_Exoneradas`, `Total_Gratuitas`, `Total_Descuento`, `Sub_Total`, `Porcentaje_IGV`, `Total_IGV`, `Total_ISC`, `Total_ICBPER`, `Total_Impuestos`, `Total_Otrs_Ompuestos`, `Total`, `Rucproveedor`, `NombreProveedor`, `DireccionProveedor`, `TotalLetras`, `restastock`, `observacion`, `Cod_Tipo_Documento`) VALUES (?,?, ?, ?, ?,?,?, ?, ?, ?,?,?, ?, ?, ?,?,?, ?, ?, ?,?,?, ?, ?, ?,?,?, ?, ?, ?,?,?)";
    $stmt_cabecera = mysqli_prepare($conn, $sql_insert_cabecera);
    mysqli_stmt_bind_param($stmt_cabecera, "iiiiisssssssssssssssssssssssssss", $ID_Sucursal, $ID_Metodo_Pago, $ID_Proveedor, $ID_Usuario, $ID_Moneda, $Forma_Pago, $NroComprobante, $Estado_Activo, $Estado_Entrega, $Estado_Canje, $Estado_Prestamo, $HoraCompra, $Fecha_Compra, $Total_Grabadas, $Total_Exoneradas, $Total_Gratuitas, $Total_Descuento, $Sub_Total, $Porcentaje_IGV, $Total_IGV, $Total_ISC, $Total_ICBPER, $Total_Impuestos, $Total_Otrs_Ompuestos, $Total, $Rucproveedor, $NombreProveedor, $DireccionProveedor, $TotalLetras, $restastock, $observacion,$Cod_Tipo_Documento);
    if (!mysqli_stmt_execute($stmt_cabecera)) {
        throw new Exception("Error al guardar los datos de la cabecera de la compra: " . mysqli_error($conn), 500);
    }
    $id_compra = mysqli_insert_id($conn); // Obtener el ID de la compra insertada 


if($Forma_Pago=="0"){
    $sql_insert_Cabecera_CreditoVentas = "INSERT INTO `Cabecera_CreditoCompras` (`ID_Compras`,`ID_Sucursal`, `ID_Usuario`, `Estado_Cancelado`,`Fecha_vence`, `Total`, `Acuenta`, `Pendiente`)VALUES 
    ('$id_compra','$ID_Sucursal','$ID_Usuario','0','$Fecha_vence','$Total','$Acuenta','$Pendiente')";

    if (!mysqli_query($conn, $sql_insert_Cabecera_CreditoVentas)) {
        throw new Exception("Error al guardar los datos de la cabecera de la Cabecera_CreditoVentas: " . mysqli_error($conn));
    }
    
    $id_caCOMPRA = mysqli_insert_id($conn);
    
    foreach ($datos['cuotas'] as $cuotas) {
      $COD_FORMA_PAGO = $cuotas['COD_FORMA_PAGO'];
      $MONTO_FORMA_PAGO = $cuotas['MONTO_FORMA_PAGO'];
      $FECHA_FORMA_PAGO = $cuotas['FECHA_FORMA_PAGO'];

        $sql_insert_metodopagoventa= "INSERT INTO `Detalle_ComprasCredito` (`ID_Cab_CreditoC`, `Fecha_emision`, `COD_FORMA_PAGO`, `MONTO_FORMA_PAGO`, `FECHA_FORMA_PAGO`) 
        VALUES 
        ('$id_caCOMPRA','$Fecha_Compra','$COD_FORMA_PAGO','$MONTO_FORMA_PAGO','$FECHA_FORMA_PAGO')";
    
        if (!mysqli_query($conn, $sql_insert_metodopagoventa)) {
            throw new Exception("Error al guardar los datos de la cabecera de la metodo pago venta: " . mysqli_error($conn));
        }
    }
}

    // Insertar los detalles de la compra
    $sql_insert_detalle = "INSERT INTO Detalle_Compras(ID_Compra, ID_Insumo, ID_Sucursal, Cantidad, Cantidad_Recogida, Precio_Unitario, Total, Estado_Entrega) VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
    $stmt_detalle = mysqli_prepare($conn, $sql_insert_detalle);
    foreach ($datos['Detalle'] as $detalle) {
        
        $ID_Insumo = mysqli_real_escape_string($conn, $detalle['ID_Producto']);
        $Cantidad = mysqli_real_escape_string($conn, $detalle['Cantidad_Vendida']);
        $Cantidad_Recogida = mysqli_real_escape_string($conn, $detalle['Cantidad_Entrega']);// Cantidad_Recogida
        $Precio_Unitario = mysqli_real_escape_string($conn, $detalle['Precio_Unitario']);
        $Estado_Entrega = mysqli_real_escape_string($conn, $detalle['Estado_Entrega']);
        $Totaldetalle = mysqli_real_escape_string($conn, $detalle['Total']);
        
        
        mysqli_stmt_bind_param($stmt_detalle, "iiisssss", $id_compra, $ID_Insumo, $ID_Sucursal, $Cantidad, $Cantidad_Recogida, $Precio_Unitario, $Totaldetalle, $Estado_Entrega);
        if (!mysqli_stmt_execute($stmt_detalle)) {
            throw new Exception("Error al guardar los datos del detalle de la compra: " . mysqli_error($conn), 500);
        }
    }
    
    mysqli_stmt_close($stmt_detalle);
    // Confirmar la transaccion
    mysqli_commit($conn);
    

    $resultado['ESTADO'] = "1";
    $resultado['MENSAJE'] = "La informacion se registro correctamente";
    $resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
    mysqli_rollback($conn);
    
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
