<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");

    $nombre_tabla = 'Cabecera_contrato';
    $tipo_permiso = 'listar';

    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporcionó el campo token
    if (!isset($datos['token'])) {
        throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para LISTAR.", 400);
    }

    // Obtener otros datos del array (asegurándose de que estén establecidos)
    $token = $datos['token'];
    $ID_Sucursal = $datos['ID_Sucursal'] ?? null;
    $Fecha_inicio = $datos['Fecha_inicio'] ?? null;
    $Fecha_fin = $datos['Fecha_fin'] ?? null;
    $ID_Usuario = $datos['ID_Usuario'] ?? null;
    $Estadoo = $datos['Estado'] ?? null;


    // Verificar si no se enviaron valores para los campos de fecha
    if (empty($Fecha_inicio) && empty($Fecha_fin)) {
        // Obtener el rango de fechas del presente mes
        $Fecha_inicio = date('Y-m-01');
        $Fecha_fin = date('Y-m-t');
    }

    // Verificar autorización del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorización inválido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para Lista Cabecera_Ventas en esta tabla.", 403);
    }

    // Construir y ejecutar la consulta SQL
    $sql = "SELECT CC.*, S.Nombre_Sucursal, U.ID_Empleado, U.correo, E.*
            FROM Cabecera_contrato CC
            JOIN Sucursales S ON CC.ID_Sucursal = S.ID_Sucursal
            JOIN usuarios U ON CC.ID_Usuario = U.ID_Empleado
            JOIN Empleados E ON U.ID_Empleado = E.ID_Empleado
            WHERE 1";

    $params = array();
    $types = '';
    if (!empty($ID_Sucursal)) {
        $sql .= " AND CC.ID_Sucursal = ?";
        $params[] = $ID_Sucursal;
        $types .= 'i';
    }

    if (!empty($ID_Usuario)) {
        $sql .= " AND CC.ID_Usuario = ?";
        $params[] = $ID_Usuario;
        $types .= 'i';
    }

    if (!empty($Fecha_inicio) && !empty($Fecha_fin)) {
        $sql .= " AND CC.Fecha_emision BETWEEN ? AND ?";
        $params[] = $Fecha_inicio;
        $params[] = $Fecha_fin;
        $types .= 'ss';
    }

    if (!empty($Estadoo)) {
        $sql .= " AND CC.Estado = ?";
        $params[] = $Estadoo;
        $types .= 'i';
    }

    $sql .= " ORDER BY CC.Fecha_emision DESC, CC.Hora_emision ASC";

    // Preparar la consulta
    $stmt = mysqli_prepare($conn, $sql);
    if ($stmt) {
        // Vincular parámetros
        if (!empty($params)) {
            mysqli_stmt_bind_param($stmt, $types, ...$params);
        }

        // Ejecutar la consulta
        if (mysqli_stmt_execute($stmt)) {
            // Obtener resultados
            $result = mysqli_stmt_get_result($stmt);
            $rows = mysqli_fetch_all($result, MYSQLI_ASSOC);
            mysqli_free_result($result);

            // Comprobar si hay datos
            if (count($rows) > 0) {
                print_json($rows, 1, "Registros disponibles.", 200);
            } else {
                print_json([], 0, "No se encontraron registros.", 400);
            }
        } else {
            throw new Exception("ERROR: Could not able to execute query. " . mysqli_error($conn));
        }

        // Cerrar la declaración
        mysqli_stmt_close($stmt);
    } else {
        throw new Exception("ERROR: Could not prepare query. " . mysqli_error($conn));
    }

} catch (Exception $e) {
    print_json([], 0, $e->getMessage());
}

mysqli_close($conn);

function print_json($data, $estado = 1, $mensaje, $http_code = 200) {
    $response = array(
        "server_mensaje" => array(
            "ESTADO" => $estado,
            "MENSAJE" => $mensaje
        ),
        "data" => $data
    );

    http_response_code($http_code);
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
