<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    // Iniciar transaccin
    mysqli_begin_transaction($conn);
    
    $nombre_tabla = 'Cabecera_contrato';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
     if (!isset($datos['ID_Sucursal'], $datos['Detalle'], $datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no est vaco y no consiste solo en espacios en blanco
    if (empty($datos['ID_Sucursal']) || empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacos o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del arrayss
    $token = $datos['token'];
    $tipo_comprobanteSeleccionado = mysqli_real_escape_string($conn, $datos['tipo_comprobanteSeleccionado']);
    $ID_Sucursal = mysqli_real_escape_string($conn, $datos['ID_Sucursal']);
    $ID_Usuario = mysqli_real_escape_string($conn, $datos['ID_Usuario']);
    $Fecha_emision = mysqli_real_escape_string($conn, $datos['Fecha_emision']); 
    $Fecha_evento = mysqli_real_escape_string($conn, $datos['Fecha_evento']); 
    $Hora_emision = mysqli_real_escape_string($conn, $datos['Hora_emision']);
    $Horaevento = mysqli_real_escape_string($conn, $datos['Horaevento']);
    $RUC_cliente = mysqli_real_escape_string($conn, $datos['RUC_cliente']);
    $Nombre_cliente = mysqli_real_escape_string($conn, $datos['Nombre_cliente']);
    $Total = mysqli_real_escape_string($conn, $datos['Total']);
    $Adelanto = mysqli_real_escape_string($conn, $datos['Adelanto']);
    $Estado = mysqli_real_escape_string($conn, $datos['Estado']);
    $Evento = mysqli_real_escape_string($conn, $datos['Evento']);
    $Telefonocliente = mysqli_real_escape_string($conn, $datos['Telefonocliente']); 
    $Tipo_Contrato = mysqli_real_escape_string($conn, $datos['Tipo_Contrato']); 

    
    
    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para editar en esta tabla.", 403);
    }


        $query = "SELECT numero,serie FROM correlativo_sucursal_BFN WHERE ID_Sucursal = '$ID_Sucursal' AND tipo_documento = '$tipo_comprobanteSeleccionado' FOR UPDATE";
        $result = mysqli_query($conn, $query);
        $row = mysqli_fetch_assoc($result);
        $numeroActual = $row['numero'];
        $serie = $row['serie'];

        // Incrementar el numero actual en 1
        $numeroNuevo = $numeroActual + 1;

        // Actualizar el numero en la tabla correlativo_sucursal_BFN
        $queryUpdate = "UPDATE correlativo_sucursal_BFN SET numero = $numeroNuevo WHERE ID_Sucursal = '$ID_Sucursal' AND tipo_documento= '$tipo_comprobanteSeleccionado'";
        if (!mysqli_query($conn, $queryUpdate)) {
            throw new Exception(mysqli_error($conn));
        }

        // Formatear el numero nuevo con ceros a la izquierda
        $numeroFormateado = str_pad($numeroNuevo, 8, '0', STR_PAD_LEFT);

        // Usar el numero formateado como nrocomprobante
        $nroComprobante = $serie . '-' . $numeroFormateado;



// Insertar la cabecera de la compra
    $sql_insert_cabecera = "INSERT INTO `Cabecera_contrato` (`ID_Sucursal`, `ID_Usuario`, `NroComprobante`, `Fecha_emision`, `Fecha_evento`, `Hora_emision`, `Horaevento`, `Tipo_Contrato`, `RUC_cliente`, `Nombre_cliente`, `Telefonocliente`, `Total`, `Adelanto`, `Estado`, `Evento`) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
    $stmt_cabecera = mysqli_prepare($conn, $sql_insert_cabecera);
    mysqli_stmt_bind_param($stmt_cabecera, "iisssssssssssss", $ID_Sucursal, $ID_Usuario, $nroComprobante, $Fecha_emision, $Fecha_evento, $Hora_emision, $Horaevento,$Tipo_Contrato, $RUC_cliente, $Nombre_cliente,$Telefonocliente, $Total, $Adelanto, $Estado, $Evento);
    if (!mysqli_stmt_execute($stmt_cabecera)) {
        throw new Exception("Error al guardar los datos de la cabecera de la proforma: " . mysqli_error($conn), 500);
    }
    $id_compra = mysqli_insert_id($conn); // Obtener el ID de la compra insertada

    // Insertar los detalles de la compra
    $sql_insert_detalle = "INSERT INTO Detalle_contrato(ID_contrato, Descripcion, Cantidad, Precio_unitario, Total, Otros1, Otros2) VALUES (?, ?, ?, ?, ?, ?, ?)";
    $stmt_detalle = mysqli_prepare($conn, $sql_insert_detalle);
    foreach ($datos['Detalle'] as $detalle) {
        
        $Descripcion = mysqli_real_escape_string($conn, $detalle['producto']);
        $Cantidad = mysqli_real_escape_string($conn, $detalle['Cantidad_Vendida']);
        $Precio_unitario = mysqli_real_escape_string($conn, $detalle['Precio_Unitario']);
        $Total = mysqli_real_escape_string($conn, $detalle['Total']);
        $Otros1 = mysqli_real_escape_string($conn, $detalle['Otros1']);
        $Otros2 = mysqli_real_escape_string($conn, $detalle['Otros2']);
        
        
        mysqli_stmt_bind_param($stmt_detalle, "issssss", $id_compra, $Descripcion, $Cantidad,  $Precio_unitario, $Total, $Otros1, $Otros2);
        if (!mysqli_stmt_execute($stmt_detalle)) {
            throw new Exception("Error al guardar los datos del detalle de la proforma: " . mysqli_error($conn), 500);
        }
    }
    
    mysqli_stmt_close($stmt_detalle);
    // Confirmar la transaccion
    mysqli_commit($conn);
    

    $resultado['COMPROBANTE'] =$nroComprobante;
    $resultado['ESTADO'] = "1";
    $resultado['MENSAJE'] = "La informacion se registro correctamente";
    $resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
    mysqli_rollback($conn);
    $resultado['COMPROBANTE'] ='0';
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
