<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");

    $nombre_tabla = 'Cabecera_contrato';
    $tipo_permiso = 'listar';

    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporcionó el campo token
    if (!isset($datos['token'])) {
        throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para LISTAR.", 400);
    }

    // Obtener otros datos del array (asegurándose de que estén establecidos)
    $token = $datos['token'];

    $ID_Sucursal = isset($datos['ID_Sucursal']) ? $datos['ID_Sucursal'] : null;
    $Fecha_inicio = isset($datos['Fecha_inicio']) ? $datos['Fecha_inicio'] : null;
    $Fecha_fin = isset($datos['Fecha_fin']) ? $datos['Fecha_fin'] : null;
    $ID_Usuario = isset($datos['ID_Usuario']) ? $datos['ID_Usuario'] : null;
    
    
        // Verificar si no se enviaron valores para los campos de fecha
    if (empty($Fecha_inicio) && empty($Fecha_fin)) {
        // Obtener el rango de fechas del presente mes
        $Fecha_inicio = date('Y-m-01');
        $Fecha_fin = date('Y-m-t');
    }
    
    // Verificar autorización del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorización inválido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para Lista Cabecera_Ventas en esta tabla.", 403);
    }

    // Construir y ejecutar las consultas SQL
    //$sql1 = "SELECT sum(Total) as total_ventas_vigentes from Cabecera_Ventas where Estado_Activo=1 AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND'$Fecha_fin'";
    $sql1 = "SELECT sum(Total) as total_ventas_vigentes from Cabecera_Ventas where Estado_Activo=1 AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";

    $sql2 = "SELECT sum(Total) as total_ventas_anuladas from Cabecera_Ventas where Estado_Activo=0 AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";
    $sql3 = "SELECT sum(Total) as total_ventas_vigentesfactura from Cabecera_Ventas where Estado_Activo=1 AND Estado_Canje=0 and Cod_Tipo_Documento=01 AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";
    $sql4 = "SELECT sum(Total) as total_ventas_anuladasfactura from Cabecera_Ventas where Estado_Activo=0 AND Cod_Tipo_Documento=01 AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";
    $sql5 = "SELECT sum(Total) as total_ventas_vigentesboleta from Cabecera_Ventas where Estado_Activo=1 AND Estado_Canje=0 and Cod_Tipo_Documento=03 AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";
    $sql6 = "SELECT sum(Total) as total_ventas_anuladasboleta from Cabecera_Ventas where Estado_Activo=0 AND Cod_Tipo_Documento=03 AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";
    $sql7 = "SELECT sum(Total) as total_ventas_vigentesnota from Cabecera_Ventas where Estado_Activo=1 AND Estado_Canje=0 and Cod_Tipo_Documento=00 AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";
    $sql8 = "SELECT sum(Total) as total_ventas_anuladasnota from Cabecera_Ventas where Estado_Activo=0 AND Cod_Tipo_Documento=00AND Estado_Canje=0  AND Fecha_Venta BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";

    $sql9 = "SELECT sum(Total) as total_gastos from Gastos AND Estado_Canje=0  AND Fecha BETWEEN '$Fecha_inicio' AND '$Fecha_fin'";
    $sql10 = "SELECT sum(Total) as total_pagosxrealizar FROM Cabecera_CreditoCompras;";
    $sql11 = "SELECT sum(Total) as total_cobrosxrealizar FROM Cabecera_CreditoCompras;";
    $sql12 = "SELECT count(Stock) totalsinstock FROM `StockProductos_PorSucursal` WHERE Stock<=0 and ID_Sucursal='$ID_Sucursal'";

    // Ejecutar las consultas
    $result1 = mysqli_query($conn, $sql1);
    $result2 = mysqli_query($conn, $sql2);
    $result3 = mysqli_query($conn, $sql3);
    $result4 = mysqli_query($conn, $sql4);
    $result5 = mysqli_query($conn, $sql5);
    $result6 = mysqli_query($conn, $sql6);
    $result7 = mysqli_query($conn, $sql7);
    $result8 = mysqli_query($conn, $sql8);

    $result9 = mysqli_query($conn, $sql9);
    $result10 = mysqli_query($conn, $sql10);
    $result11 = mysqli_query($conn, $sql11);
    $result12 = mysqli_query($conn, $sql12);
    
    // Obtener los resultados
    $total_ventas_vigentes = mysqli_fetch_assoc($result1) ?? array("total_ventas_vigentes" => "0");
    $total_ventas_anuladas = mysqli_fetch_assoc($result2) ?? array("total_ventas_anuladas" => "0");
    $total_ventas_vigentesfactura = mysqli_fetch_assoc($result3) ?? array("total_ventas_vigentesfactura" => "0");
    $total_ventas_anuladasfactura = mysqli_fetch_assoc($result4) ?? array("total_ventas_anuladasfactura" => "0");
    $total_ventas_vigentesboleta = mysqli_fetch_assoc($result5) ?? array("total_ventas_vigentesboleta" => "0");
    $total_ventas_anuladasboleta = mysqli_fetch_assoc($result6) ?? array("total_ventas_anuladasboleta" => "0");
    $total_ventas_vigentesnota = mysqli_fetch_assoc($result7) ?? array("total_ventas_vigentesnota" => "0");
    $total_ventas_anuladasnota = mysqli_fetch_assoc($result8) ?? array("total_ventas_anuladasnota" => "0");

    $total_gastos = mysqli_fetch_assoc($result9) ?? array("total_gastos" => "0");
    $total_pagosxrealizar = mysqli_fetch_assoc($result10) ?? array("total_pagosxrealizar" => "0");
    $total_cobrosxrealizar = mysqli_fetch_assoc($result11) ?? array("total_cobrosxrealizar" => "0");
    $totalsinstock = mysqli_fetch_assoc($result12) ?? array("totalsinstock" => "0");

    $data = array(
        "total_ventas_vigentes" => $total_ventas_vigentes,
        "total_ventas_anuladas" => $total_ventas_anuladas,
        "total_ventas_vigentesfactura" => $total_ventas_vigentesfactura,
        "total_ventas_anuladasfactura" => $total_ventas_anuladasfactura,
        "total_ventas_vigentesboleta" => $total_ventas_vigentesboleta,
        "total_ventas_anuladasboleta" => $total_ventas_anuladasboleta,
        "total_ventas_vigentesnota" => $total_ventas_vigentesnota,
        "total_ventas_anuladasnota" => $total_ventas_anuladasnota,
        
        "total_gastos" => $total_gastos,
        "total_pagosxrealizar" => $total_pagosxrealizar,
        "total_cobrosxrealizar" => $total_cobrosxrealizar,
        "totalsinstock" => $totalsinstock
    );

    print_json($data, 1, "Registros disponibles.");

} catch (Exception $e) {
    print_json([], 0, $e->getMessage());
}

mysqli_close($conn);

function print_json($data, $estado = 1, $mensaje, $http_code = 200) {
    $response = array(
        "server_mensaje" => array(
            "ESTADO" => $estado,
            "MENSAJE" => $mensaje
        ),
        "data" => $data
    );

    http_response_code($http_code);
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
