<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

$bodyRequest = file_get_contents("php://input");

// Decodifica el cuerpo de la solicitud y lo guarda en un array de PHP
$datos = json_decode($bodyRequest, true);

$cod_usuario = isset($datos['ID_Usuario']) ? intval($datos['ID_Usuario']) : 0;
$cod_sucursal_empresa = isset($datos['ID_Sucursal']) ? intval($datos['ID_Sucursal']) : 0;

// Obtener el año actual
$anio_actual = date('Y');

// Construir la consulta SQL base
$sql = "SELECT MONTH(Fecha_Venta) AS mes, SUM(total) AS total
        FROM Cabecera_Ventas
        WHERE YEAR(Fecha_Venta) = ?";

// Verificar las combinaciones de parámetros
if (!empty($cod_sucursal_empresa)) {
    $sql .= " AND ID_Sucursal = ?";
}

if (!empty($cod_usuario)) {
    $sql .= " AND ID_Usuario = ?";
}

$sql .= " GROUP BY mes
          ORDER BY mes";

$stmt = mysqli_prepare($conn, $sql);

// Establecer los parámetros dinámicamente en función de los valores proporcionados
$params = array('i', $anio_actual);

if (!empty($cod_sucursal_empresa)) {
    $params[] = $cod_sucursal_empresa;
    $params[0] .= 's';
}

if (!empty($cod_usuario)) {
    $params[] = $cod_usuario;
    $params[0] .= 's';
}

// Hacer binding de los parámetros
call_user_func_array('mysqli_stmt_bind_param', array_merge(array($stmt), $params));

mysqli_stmt_execute($stmt);
$result = mysqli_stmt_get_result($stmt);

$ventas_por_mes = array_fill(1, 12, 0); // Inicializa un array con ceros para todos los meses

if ($result) {
    while ($row = mysqli_fetch_assoc($result)) {
        $mes = (int)$row['mes'];
        $total = (float)$row['total'];
        $ventas_por_mes[$mes] = $total;
    }
    mysqli_free_result($result);
}

mysqli_close($conn);

// Mapear los números de mes a nombres de mes
$meses_nombres = array(
    1 => 'enero',
    2 => 'febrero',
    3 => 'marzo',
    4 => 'abril',
    5 => 'mayo',
    6 => 'junio',
    7 => 'julio',
    8 => 'agosto',
    9 => 'septiembre',
    10 => 'octubre',
    11 => 'noviembre',
    12 => 'diciembre'
);

// Crear un nuevo array con los nombres de los meses
$ventas_por_mes_nombres = array();
foreach ($ventas_por_mes as $mes => $total) {
    $nombre_mes = $meses_nombres[$mes];
    $ventas_por_mes_nombres[$nombre_mes] = $total;
}

// Formatear el resultado como JSON
$response = array(
    'ESTADO' => "1",
    'MENSAJE' => "Registros disponibles.",
    'ventas_por_mes' => $ventas_por_mes_nombres
);

header("Content-type: application/json; charset=utf-8");
echo json_encode($response, JSON_UNESCAPED_UNICODE);
?>
