<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

$bodyRequest = file_get_contents("php://input");
date_default_timezone_set("America/Lima");

// Decodifica el cuerpo de la solicitud y lo guarda en un array de PHP
$datos = json_decode($bodyRequest, true);
$fecha_actual_peru = date('Y-m-d');

$cod_usuario = isset($datos['cod_usuario']) ? $datos['cod_usuario'] : "";
$cod_sucursal_empresa = isset($datos['cod_sucursal_empresa']) ? $datos['cod_sucursal_empresa'] : "";

// Obtener el año y la semana actual
$anio_actual = date('Y');
$semana_actual = date('W');

// Construir la consulta SQL base
$sql = "SELECT DAYOFWEEK(fecha_documento) AS dia_semana, SUM(total) AS total
        FROM historia_clienteperu
        WHERE YEAR(fecha_documento) = ? AND WEEK(fecha_documento, 1) = ?";

/*
$sql = "SELECT DAYOFWEEK(fecha_documento) AS dia_semana, SUM(total) AS total
        FROM historia_clienteperu
        WHERE YEAR(fecha_documento) = ? AND WEEK(fecha_documento, 7) = ?"; // Cambio en esta línea
*/

// Verificar las combinaciones de parámetros
if (!empty($cod_sucursal_empresa)) {
    $sql .= " AND cod_sucursal_empresa = ?";
}

if (!empty($cod_usuario)) {
    $sql .= " AND cod_usuario = ?";
}

$sql .= " GROUP BY dia_semana
          ORDER BY dia_semana";

$stmt = mysqli_prepare($conn, $sql);

// Establecer los parámetros dinámicamente en función de los valores proporcionados
$params = array('is', $anio_actual, $semana_actual);

if (!empty($cod_sucursal_empresa)) {
    $params[] = $cod_sucursal_empresa;
    $params[0] .= 's';
}

if (!empty($cod_usuario)) {
    $params[] = $cod_usuario;
    $params[0] .= 's';
}

// Hacer binding de los parámetros
call_user_func_array('mysqli_stmt_bind_param', array_merge(array($stmt), $params));

mysqli_stmt_execute($stmt);
$result = mysqli_stmt_get_result($stmt);

$ventas_por_dia_semana = array_fill(1, 7, 0); // Inicializa un array con ceros para todos los días de la semana

if ($result) {
    while ($row = mysqli_fetch_assoc($result)) {
        $dia_semana = (int)$row['dia_semana'];
        $total = (float)$row['total'];
        $ventas_por_dia_semana[$dia_semana] = $total;
    }
    mysqli_free_result($result);
}

mysqli_close($conn);

// Mapear los números de día de la semana a nombres de día de la semana
$dias_semana_nombres = array(
    1 => 'lunes',
    2 => 'martes',
    3 => 'miercoles',
    4 => 'jueves',
    5 => 'viernes',
    6 => 'sabado',
    7 => 'domingo'
);

// Crear un nuevo array con los nombres de los días de la semana
$ventas_por_dia_semana_nombres = array();
foreach ($ventas_por_dia_semana as $dia_semana => $total) {
    $nombre_dia_semana = $dias_semana_nombres[$dia_semana];
    $ventas_por_dia_semana_nombres[$nombre_dia_semana] = $total;
}

// Formatear el resultado como JSON
$response = array(
    'success' => true,
    'message' => "Records retrieved successfully.",
    'ventas_por_dia_semana' => $ventas_por_dia_semana_nombres
);

header("Content-type: application/json; charset=utf-8");
echo json_encode($response, JSON_UNESCAPED_UNICODE);
?>
