<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    // Iniciar transacción
    mysqli_begin_transaction($conn);
    
    $nombre_tabla = 'Insumos';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
    if (!isset($datos['Nombre_Insumo'], $datos['Precio_Unitario'], $datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['Nombre_Insumo']) || empty($datos['Precio_Unitario']) || empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array ,  Datos para la tabla Insumos
    $token = $datos['token'];
    $Foto = "https://lesaffre.com.pe/wp-content/uploads/2020/09/Timeline_6B_OtrosInsumos.jpg";
    $Nombre_Insumo = mysqli_real_escape_string($conn, $datos['Nombre_Insumo']);
    $Precio_Unitario = mysqli_real_escape_string($conn, $datos['Precio_Unitario']);
    $Unidad_Principal = isset($datos['Unidad_Principal']) ? mysqli_real_escape_string($conn, $datos['Unidad_Principal']) : '';
    $ID_Unidad = isset($datos['ID_Unidad']) ? mysqli_real_escape_string($conn, $datos['ID_Unidad']) : '';
    $Factor_Conversion = isset($datos['Factor_Conversion']) ? mysqli_real_escape_string($conn, $datos['Factor_Conversion']) : '';
    $ID_Sucursal = isset($datos['ID_Sucursal']) ? mysqli_real_escape_string($conn, $datos['ID_Sucursal']) : '';
    $Cantidad = isset($datos['Cantidad']) ? mysqli_real_escape_string($conn, $datos['Cantidad']) : '';
    
    
    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para editar en esta tabla.", 403);
    }

    // Insertar en la tabla Insumos
    $sql_insert_insumos = "INSERT INTO Insumos(Nombre_Insumo, Unidad_Principal, Precio_Unitario, Foto) VALUES ('$Nombre_Insumo', '$Unidad_Principal', '$Precio_Unitario','$Foto')";
    if (!mysqli_query($conn, $sql_insert_insumos)) {
        throw new Exception("Error al guardar los datos de Insumos: " . mysqli_error($conn), 500);
    }
    $id_insumo_insertado = mysqli_insert_id($conn);


    // Insertar en la tabla Insumos_Unidades
    $sql_insumo_unidades = "INSERT INTO Insumos_Unidades(ID_Insumo, ID_Unidad, Factor_Conversion) VALUES ('$id_insumo_insertado', '$ID_Unidad', '$Factor_Conversion')";
    if (!mysqli_query($conn, $sql_insumo_unidades)) {
        throw new Exception("Error al guardar los datos de Insumos_Unidades: " . mysqli_error($conn), 500);
    }
    $id_insumo_unidades = mysqli_insert_id($conn);


    // Insertar en la tabla Stock_Insumos_Sucursal
    $sql_stock_insumos_sucursal = "INSERT INTO Stock_Insumos_Sucursal(ID_Sucursal, ID_Insumo, Cantidad) VALUES ('$ID_Sucursal', '$id_insumo_insertado', '$Cantidad')";
    if (!mysqli_query($conn, $sql_stock_insumos_sucursal)) {
        throw new Exception("Error al guardar los datos de Stock_Insumos_Sucursal: " . mysqli_error($conn), 500);
    }
    
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    mysqli_commit($conn);
    
    // Preparar respuesta
    $resultado['ID_INSUMO'] = $id_insumo_insertado;
    $resultado['ID_INSUMO_UNIDADES'] = $id_insumo_unidades; 
    $resultado['ID_INSUMO_SUCURSALES'] = mysqli_insert_id($conn);
    $resultado['ESTADO'] = "1";
    $resultado['MENSAJE'] = "La información se guardó correctamente";
    $resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
    mysqli_rollback($conn);
    
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
