<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");

    $nombre_tabla = 'Cabecera_produccion';
    $tipo_permiso = 'listar';

    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    $datos = json_decode($bodyRequest, true);
    if ($datos === null && json_last_error() !== JSON_ERROR_NONE) {
        throw new Exception("Error al decodificar los datos JSON: " . json_last_error_msg(), 400);
    }

    if (!isset($datos['token'])) {
        throw new Exception("Faltan datos requeridos.", 400);
    }

    if (empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para LISTAR.", 400);
    }

    $token = isset($datos['token']) ? $datos['token'] : null;
    if (empty($token)) {
        throw new Exception("Token de autorización inválido.", 400);
    }

    $ID_Sucursal = isset($datos['ID_Sucursal']) ? $datos['ID_Sucursal'] : null;
    $Fecha_inicio = isset($datos['Fecha_inicio']) ? $datos['Fecha_inicio'] : null;
    $Fecha_fin = isset($datos['Fecha_fin']) ? $datos['Fecha_fin'] : null;
    $ID_Usuario = isset($datos['ID_Usuario']) ? $datos['ID_Usuario'] : null;

    if (empty($Fecha_inicio) && empty($Fecha_fin)) {
        $Fecha_inicio = date('Y-m-01');
        $Fecha_fin = date('Y-m-t');
    }

    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorización inválido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para Lista Cabecera_produccion en esta tabla.", 403);
    }

    $sql = "SELECT * FROM Cabecera_produccion WHERE 1";

    $params = array();
    $types = '';
    if (!empty($ID_Sucursal)) {
        $sql .= " AND ID_Sucursal = ?";
        $params[] = $ID_Sucursal;
        $types .= 'i';
    }

    if (!empty($ID_Usuario)) {
        $sql .= " AND ID_Usuario = ?";
        $params[] = $ID_Usuario;
        $types .= 'i';
    }

    if (!empty($Fecha_inicio) && !empty($Fecha_fin)) {
        $sql .= " AND Fecha_emision BETWEEN ? AND ?";
        $params[] = $Fecha_inicio;
        $params[] = $Fecha_fin;
        $types .= 'ss';
    }

    $sql .= " ORDER BY Fecha_emision DESC";

    $stmt = mysqli_prepare($conn, $sql);
    if ($stmt) {
        if (!empty($params)) {
            mysqli_stmt_bind_param($stmt, $types, ...$params);
        }

        if (mysqli_stmt_execute($stmt)) {
            $result = mysqli_stmt_get_result($stmt);
            $rows = mysqli_fetch_all($result, MYSQLI_ASSOC);
            mysqli_free_result($result);

            if (count($rows) > 0) {
                print_json($rows, 1, "Registros disponibles.", 200);
            } else {
                print_json([], 0, "No se encontraron registros.", 400);
            }
        } else {
            throw new Exception("ERROR: Could not able to execute query. " . mysqli_error($conn));
        }

        mysqli_stmt_close($stmt);
    } else {
        throw new Exception("ERROR: Could not prepare query. " . mysqli_error($conn));
    }

} catch (Exception $e) {
    print_json([], 0, $e->getMessage());
}

mysqli_close($conn);

function print_json($data, $estado = 1, $mensaje, $codigo = 200) {
    $response = array(
        "server_mensaje" => array(
            "ESTADO" => $estado,
            "MENSAJE" => $mensaje
        ),
        "data" => $data
    );

    http_response_code($codigo);
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
