<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
        // Iniciar transacción
    mysqli_begin_transaction($conn);
    
    $nombre_tabla = 'Productos';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
    if (!isset($datos['Nombre_Producto'], $datos['ID_Unidad'], $datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['Nombre_Producto']) || empty($datos['token']) || empty($datos['ID_Unidad']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array
    $token = $datos['token'];
    $Foto1 = "https://jumpseller.mx/generated/images/learn/los-10-productos-mas-vendidos-en-mexico/online-shopping-mexico-800-3423d44e0.png";
    $Foto2 = "https://jumpseller.mx/generated/images/learn/los-10-productos-mas-vendidos-en-mexico/online-shopping-mexico-800-3423d44e0.png";
    $Foto3 = "https://jumpseller.mx/generated/images/learn/los-10-productos-mas-vendidos-en-mexico/online-shopping-mexico-800-3423d44e0.png";
    
    $Nombre_Producto = mysqli_real_escape_string($conn, $datos['Nombre_Producto']);
    $Estado_Activo = mysqli_real_escape_string($conn, $datos['Estado_Activo']);
    $Gratuito = mysqli_real_escape_string($conn, $datos['Gratuito']);
    $afecto_IGV = mysqli_real_escape_string($conn, $datos['afecto_IGV']);
    $Porcentaje_IGV = mysqli_real_escape_string($conn, $datos['Porcentaje_IGV']);
    $Stock = mysqli_real_escape_string($conn, $datos['Stock']);
    $ID_Sucursal = mysqli_real_escape_string($conn, $datos['ID_Sucursal']);
    $ID_Unidad = mysqli_real_escape_string($conn, $datos['ID_Unidad']);
    //$Factor_Conversion = mysqli_real_escape_string($conn, $datos['Factor_Conversion']);
    $ID_Categoria = mysqli_real_escape_string($conn, $datos['ID_Categoria']);

    $Factor_Conversion = (isset($datos['Factor_Conversion']) && $datos['Factor_Conversion'] !== null && $datos['Factor_Conversion'] !== '' && $datos['Factor_Conversion'] !== '0') ? mysqli_real_escape_string($conn, $datos['Factor_Conversion']) : '1';

    $Precio_Ingreso = isset($datos['Precio_Ingreso']) && $datos['Precio_Ingreso'] !== null && $datos['Precio_Ingreso'] !== '' ? mysqli_real_escape_string($conn, $datos['Precio_Ingreso']) : '0';
    $Precio_Unitario = isset($datos['Precio_Unitario']) && $datos['Precio_Unitario'] !== null && $datos['Precio_Unitario'] !== '' ? mysqli_real_escape_string($conn, $datos['Precio_Unitario']) : '0';
    $Precio_Unitario2 = isset($datos['Precio_Unitario2']) && $datos['Precio_Unitario2'] !== null && $datos['Precio_Unitario2'] !== '' ? mysqli_real_escape_string($conn, $datos['Precio_Unitario2']) : '0';
    $Precio_Unitario3 = isset($datos['Precio_Unitario3']) && $datos['Precio_Unitario3'] !== null && $datos['Precio_Unitario3'] !== '' ? mysqli_real_escape_string($conn, $datos['Precio_Unitario3']) : '0';

        
    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para editar en esta tabla.", 403);
    }

    // Construir y ejecutar la consulta SQL
    $sql_insert_receta = "INSERT INTO Productos(Nombre_Producto,ID_Sucursal,ID_Categoria,Estado_Activo,Gratuito,afecto_IGV,Porcentaje_IGV) VALUES ('$Nombre_Producto','$ID_Sucursal','$ID_Categoria','$Estado_Activo','$Gratuito','$afecto_IGV','$Porcentaje_IGV')";
    if (!mysqli_query($conn, $sql_insert_receta)) {
        throw new Exception("Error al guardar los datos de la Productos: " . mysqli_error($conn));
    } else {
        $ID_Productos = mysqli_insert_id($conn);
    }
    
    
    $sql_Productos_Unidades = "INSERT INTO Productos_Unidades(ID_Producto,ID_Unidad,Factor_Conversion ) VALUES ('$ID_Productos','$ID_Unidad','$Factor_Conversion')";
    if (!mysqli_query($conn, $sql_Productos_Unidades)) {
        throw new Exception("Error al guardar los datos de la Productos_Unidades: " . mysqli_error($conn));
    } else {
        $ID_Productos_Unidades = mysqli_insert_id($conn);
    }
    
    
    $sql_StockProductos_PorSucursal = "INSERT INTO StockProductos_PorSucursal(ID_Producto, ID_Sucursal, Stock) VALUES ('$ID_Productos', '$ID_Sucursal','$Stock')";
    if (!mysqli_query($conn, $sql_StockProductos_PorSucursal)) {
        throw new Exception("Error al guardar los datos de la Receta: " . mysqli_error($conn));
    } else {
        $ID_Receta = mysqli_insert_id($conn);
    }
    
    $sql_Productos_foto = "INSERT INTO Producto_Foto(ID_Producto, Foto1, Foto2, Foto3) VALUES ('$ID_Productos', '$Foto1','$Foto2','$Foto3')";
    if (!mysqli_query($conn, $sql_Productos_foto)) {
        throw new Exception("Error al guardar los datos de la Producto_Foto: " . mysqli_error($conn));
    } else {
        $ID_Receta = mysqli_insert_id($conn);
    }
    
    
    $sql_insert_detalle = "INSERT INTO Precios_Productos_Sucursal(ID_Producto, ID_Sucursal, Precio_Unitario,Precio_Unitario2,Precio_Unitario3) VALUES ('$ID_Productos', '$ID_Sucursal', '$Precio_Unitario','$Precio_Unitario2','$Precio_Unitario3')";
    if (!mysqli_query($conn, $sql_insert_detalle)) {
            throw new Exception("Error al guardar los datos Precios_Productos_Sucursal: " . mysqli_error($conn));
    }
    
    
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    mysqli_commit($conn);

    $resultado['ESTADO'] = "1";
    $resultado['MENSAJE'] = "La informacion se registro correctamente";
    $resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
    mysqli_rollback($conn);
    
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    //mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    //mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
