<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.");
    }

    $nombre_tabla = 'Produccion';
    $tipo_permiso = 'insertar';

    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.");
    }

    if (!isset($datos['Detalle'], $datos['token'])) {
        throw new Exception("Faltan datos requeridos.");
    }

    $id_usuario = obtenerIdUsuarioPorToken($conn, $datos['token']);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.");
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.");
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para insertar en esta tabla.");
    }




    // Iniciar la transacción
    //mysqli_autocommit($conn, false);

    $Nombre_Producto = mysqli_real_escape_string($conn, $datos['Nombre_Producto']);
   // $Stock = mysqli_real_escape_string($conn, $datos['Stock']);
   // $ID_Sucursal = mysqli_real_escape_string($conn, $datos['ID_Sucursal']);
   // $ID_Unidad = mysqli_real_escape_string($conn, $datos['ID_Unidad']);
   // $Factor_Conversion = mysqli_real_escape_string($conn, $datos['Factor_Conversion']);

    
        $sql_insert_receta = "SELECT p.Nombre_Producto, r.Nombre_Receta, SUM(pr.Cantidad_Producida) AS Total_Cantidad_Producida
                              FROM Produccion pr  
                              JOIN Productos p ON pr.ID_Producto = p.ID_Producto  
                              JOIN Recetas r ON pr.ID_Receta = r.ID_Receta  
                              GROUP BY p.ID_Producto";
        
        $resultado_query = mysqli_query($conn, $sql_insert_receta);
        
        if (!$resultado_query) {
            throw new Exception("Error al realizar la consulta: " . mysqli_error($conn));
        } else {
            $resultado = [];
            while ($fila = mysqli_fetch_assoc($resultado_query)) {
                $resultado[] = $fila;
            }
            // Liberar el resultado
            mysqli_free_result($resultado_query);
        }

// $resultado ahora contiene los resultados de la consulta

    
    
    

} catch (Exception $e) {
    // En caso de error, hacer rollback de la transacción
   // mysqli_rollback($conn);

    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
}

print_json($resultado);
mysqli_close($conn);

function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    //mysqli_stmt_close($stmt);
    return $idUsuario;
}

function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
   // mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
