<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    
    // Iniciar transaccion
    //mysqli_begin_transaction($conn);
    
	$nombre_tabla = 'Recetas';
    $tipo_permiso = 'listar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
     if (!isset($datos['token']) || !isset($datos['ID_Receta'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no esta vacio y no consiste solo en espacios en blanco
    if (empty($datos['token']) || empty($datos['ID_Receta']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacios o con espacios en blanco, llene datos para LISTAR.", 400);
    }


    // Obtener otros datos del array
    $token = $datos['token'];
    $ID_Receta = $datos['ID_Receta'];


    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para Lista Clientes en esta tabla.", 403);
    }

    $sql = "SELECT Recetas_Insumos.*, Insumos.Nombre_Insumo, Unidades_Medida.* FROM Recetas_Insumos 
    INNER JOIN Insumos ON Recetas_Insumos.ID_Insumo = Insumos.ID_Insumo 
    INNER JOIN Unidades_Medida ON Recetas_Insumos.ID_Unidad_Medida = Unidades_Medida.ID_Unidad WHERE Recetas_Insumos.ID_Receta = ?";

    $stmt = mysqli_prepare($conn, $sql);
    if ($stmt) {
        mysqli_stmt_bind_param($stmt, 'i', $ID_Receta);
        if (mysqli_stmt_execute($stmt)) {
            $result = mysqli_stmt_get_result($stmt);
            $rows = mysqli_fetch_all($result, MYSQLI_ASSOC);
            mysqli_free_result($result);

            if (count($rows) > 0) {
                print_json($rows, 1, "Registros disponibles.", 200);
            } else {
                print_json([], 0, "No se encontraron registros.", 404);
            }
        } else {
            throw new Exception("ERROR: No se pudo ejecutar la consulta. " . mysqli_error($conn), 500);
        }
        mysqli_stmt_close($stmt);
    } else {
        throw new Exception("ERROR: No se pudo preparar la consulta. " . mysqli_error($conn), 500);
    }
    
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    //mysqli_commit($conn);

} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
    //mysqli_rollback($conn);

    print_json([], 0, $e->getMessage());
}

mysqli_close($conn);

function print_json($data, $estado = 1, $mensaje) {
    $response = array(
        "server_mensaje" => array(
            "ESTADO" => $estado,
            "MENSAJE" => $mensaje
        ),
        "data" => $data
    );

    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
