<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
        // Iniciar transacción
    mysqli_begin_transaction($conn);
    
    $nombre_tabla = 'Recetas';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
    if (!isset($datos['Nombre_Receta'], $datos['Detalle'], $datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['Nombre_Receta']) || empty($datos['token']) || trim($datos['Nombre_Receta']) === '' || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array
    $token = $datos['token'];
    $Nombre_Receta = mysqli_real_escape_string($conn, $datos['Nombre_Receta']);

    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para editar en esta tabla.", 403);
    }

    $sql_insert_receta = "INSERT INTO Recetas(Nombre_Receta) VALUES ('$Nombre_Receta')";
    if (!mysqli_query($conn, $sql_insert_receta)) {
        throw new Exception("Error al guardar los datos de la Receta: " . mysqli_error($conn));
    } else {
        $ID_Receta = mysqli_insert_id($conn);
    }
    
    foreach ($datos['Detalle'] as $detalle) {
        $Cantidad_Usada = mysqli_real_escape_string($conn, $detalle['Cantidad_Usada']);
        $ID_Insumo = mysqli_real_escape_string($conn, $detalle['ID_Insumo']);
        $ID_unidad_medida = mysqli_real_escape_string($conn, $detalle['ID_unidad_medida']);
        $Equivalencia = mysqli_real_escape_string($conn, $detalle['Equivalencia']);
        
        $sql_insert_detalle = "INSERT INTO Recetas_Insumos(ID_Receta, ID_Insumo,ID_unidad_medida, Cantidad_Usada,Equivalencia) VALUES ('$ID_Receta', '$ID_Insumo', '$ID_unidad_medida', '$Cantidad_Usada', '$Equivalencia')";
    
        if (!mysqli_query($conn, $sql_insert_detalle)) {
            throw new Exception("Error al guardar los datos del detalle de la Receta-Insumos: " . mysqli_error($conn));
        }
    }
    
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    mysqli_commit($conn);

    $resultado['ESTADO'] = "1";
    $resultado['MENSAJE'] = "La informacion se registro correctamente";
    $resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
    mysqli_rollback($conn);
    
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
