<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    
    // Iniciar transacción
    //mysqli_begin_transaction($conn);
    $nombre_tabla = 'Recetas_Insumos';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo ID_Receta, ID_Insumo y token
    if (!isset($datos['ID_Receta'], $datos['ID_Insumo'], $datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo ID_Receta, ID_Insumo y token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['ID_Receta']) || empty($datos['ID_Insumo']) || empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para registro.", 400);
    }

    // Obtener otros datos del array
    $token = $datos['token'];
    $ID_Receta = mysqli_real_escape_string($conn, $datos['ID_Receta']);
    $ID_Insumo = mysqli_real_escape_string($conn, $datos['ID_Insumo']);
    $ID_unidad_medida = isset($datos['ID_unidad_medida']) ? mysqli_real_escape_string($conn, $datos['ID_unidad_medida']) : '';
    $Cantidad_Usada = isset($datos['Cantidad_Usada']) ? mysqli_real_escape_string($conn, $datos['Cantidad_Usada']) : '';
    $Equivalencia = isset($datos['Equivalencia']) ? mysqli_real_escape_string($conn, $datos['Equivalencia']) : '';
    

    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para insertar en esta tabla.", 403);
    }

    
        $sql_insert_sucursal = "INSERT INTO Recetas_Insumos(ID_Receta, ID_Insumo, ID_unidad_medida, Cantidad_Usada, Equivalencia) VALUES (?, ?, ?, ?, ?)";
        $stmt = mysqli_prepare($conn, $sql_insert_sucursal);
        mysqli_stmt_bind_param($stmt, "sssss", $ID_Receta, $ID_Insumo, $ID_unidad_medida, $Cantidad_Usada, $Equivalencia);
        if (!mysqli_stmt_execute($stmt)) {
            throw new Exception("Error al guardar los datos de la Recetas_Insumos: " . mysqli_error($conn), 500);
        }
        mysqli_stmt_close($stmt);
    
        // Obtener el ID del cliente insertado
        $id_cliente_insertado = mysqli_insert_id($conn);
    
        $resultado['ID_Cliente'] = $id_cliente_insertado;
        $resultado['ESTADO'] = "1";
        $resultado['MENSAJE'] = "El item se registró correctamente.";
        $resultado['CODIGO'] = "200";
    
    
    // Confirmar la transaccion
    //mysqli_commit($conn);

 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
   // mysqli_rollback($conn);
    
    $resultado['ID_Cliente'] = "0";
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
