<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    
        // Iniciar transacción
    //mysqli_begin_transaction($conn);
    $nombre_tabla = 'Asistencias';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token 
    if (!isset($datos['ID_Empleado'], $datos['Fecha'], $datos['tipo_registro'], $datos['token'])) { 
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['ID_Empleado']) ||empty($datos['tipo_registro']) ||empty($datos['Fecha']) || empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array
    $token = $datos['token'];
    $tipo_registro = isset($datos['tipo_registro']) ? mysqli_real_escape_string($conn, $datos['tipo_registro']) : '';
    $ID_Empleado = isset($datos['ID_Empleado']) ? mysqli_real_escape_string($conn, $datos['ID_Empleado']) : '';
    $Fecha = isset($datos['Fecha']) ? mysqli_real_escape_string($conn, $datos['Fecha']) : '';
    $Hora_Entrada = isset($datos['Hora_Entrada']) ? mysqli_real_escape_string($conn, $datos['Hora_Entrada']) : '';
    $Horas_Tarde =isset($datos['Horas_Tarde']) ? mysqli_real_escape_string($conn, $datos['Horas_Tarde']) : '';
    $Hora_Salida =isset($datos['Hora_Salida']) ? mysqli_real_escape_string($conn, $datos['Hora_Salida']) : '';
    $Horas_Extra =isset($datos['Horas_Extra']) ? mysqli_real_escape_string($conn, $datos['Horas_Extra']) : '';
    $Estado_Asistencia ='-'; 
 

    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para insertar en esta tabla.", 403);
    }
    
    // Construir y ejecutar la consulta SQL
    if ($tipo_registro==='salida') {
        $sql_update_asistencia= "UPDATE Asistencias SET Hora_Salida=? WHERE Fecha= ? and ID_Empleado=?";
        $stmt = mysqli_prepare($conn, $sql_update_asistencia);
        mysqli_stmt_bind_param($stmt, "ssi", $Hora_Salida, $Fecha, $ID_Empleado);
        if (!mysqli_stmt_execute($stmt)) {
            throw new Exception("Error al actualizar salida de asistencia: " . mysqli_stmt_error($stmt), 500);
        }
    }else{
        $Hora_Salida ='00:00:00';
        // Verificar si ya existe un registro para hoy e ID_Empleado
        $sql_verificar_registro = "SELECT COUNT(*) FROM Asistencias WHERE Fecha = ? AND ID_Empleado = ?";
        $stmt_verificar = mysqli_prepare($conn, $sql_verificar_registro);
        mysqli_stmt_bind_param($stmt_verificar, "si", $Fecha, $ID_Empleado);
        mysqli_stmt_execute($stmt_verificar);
        mysqli_stmt_bind_result($stmt_verificar, $cantidad_registros);
        mysqli_stmt_fetch($stmt_verificar);
        mysqli_stmt_close($stmt_verificar);
        
        if ($cantidad_registros > 0) {
            $resultado['ESTADO'] = "0";
            $resultado['MENSAJE'] = "No es posible marcar doble asistencia en un mismo dia";
            $resultado['CODIGO'] = "409"; // Código de conflicto
        } else {
            // No existe un registro para hoy e ID_Empleado, proceder con la inserción
            $sql_insert_sucursal = "INSERT INTO Asistencias(ID_Empleado, Fecha, Hora_Entrada, Horas_Tarde, Hora_Salida, Horas_Extra, Estado_Asistencia) VALUES (?, ?, ?, ?, ?, ?,?)";
            $stmt = mysqli_prepare($conn, $sql_insert_sucursal);
            mysqli_stmt_bind_param($stmt, "issssss", $ID_Empleado, $Fecha, $Hora_Entrada, $Horas_Tarde, $Hora_Salida, $Horas_Extra, $Estado_Asistencia);
            if (!mysqli_stmt_execute($stmt)) {
                throw new Exception("Error al guardar los datos de la ingreso de Asistencias: " . mysqli_error($conn), 500);
            }
            // Aquí podrías establecer el mensaje de éxito si lo deseas
            $resultado['ESTADO'] = "1";
            $resultado['MENSAJE'] = "La asistencia fue registrado exitosamente";
            $resultado['CODIGO'] = "200"; // Código OK
        }
        
    }

    


    
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    //mysqli_commit($conn);

    //$resultado['ESTADO'] = "1";
    //$resultado['MENSAJE'] = "La informacion se registro correctamente";
    //$resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
   // mysqli_rollback($conn);
    
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
