<?php

error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

// Obtener datos del cuerpo de la solicitud
$bodyRequest = file_get_contents("php://input");

// Decodificar el cuerpo de la solicitud y guardarlo en un array de PHP
$datos = json_decode($bodyRequest, true);

// Verificar si la solicitud es del tipo POST
if ($_SERVER["REQUEST_METHOD"] === "POST") {
    // Validar datos
    if (!empty($datos['correo']) && !empty($datos['contrasena'])) {
        // Filtrar y validar datos
        $username = mysqli_real_escape_string($conn, $datos['correo']);

        // Preparar la consulta con consultas preparadas
        $sql = "SELECT id, contrasena FROM usuarios WHERE correo = ?";
        $stmt = mysqli_prepare($conn, $sql);

        // Vincular parámetro
        mysqli_stmt_bind_param($stmt, "s", $username);

        // Ejecutar la consulta
        mysqli_stmt_execute($stmt);

        // Vincular resultados
        mysqli_stmt_bind_result($stmt, $id, $contrasena);

        // Obtener resultados
        mysqli_stmt_fetch($stmt);

        // Cerrar la consulta preparada de selección
        mysqli_stmt_close($stmt);

        // Verificar la contraseña
        if ($id && password_verify($datos['contrasena'], $contrasena)) {
            // Generar un token de sesión único
            $token = bin2hex(random_bytes(16));

            // Insertar el token de sesión en la tabla 'sesiones'
            $sql_insert_session = "INSERT INTO sesiones (id_usuario, token) VALUES (?, ?)";
            $stmt_insert_session = mysqli_prepare($conn, $sql_insert_session);
            mysqli_stmt_bind_param($stmt_insert_session, "is", $id, $token);
            mysqli_stmt_execute($stmt_insert_session);

            // Verificar si la inserción fue exitosa
            if (mysqli_stmt_affected_rows($stmt_insert_session) > 0) {
                // Inserción exitosa
                $resultado['ESTADO'] = "1";
                $resultado['MENSAJE'] = "Inicio de sesión exitoso";
                $resultado['TOKEN_SESION'] = $token;
            } else {
                // Error en la inserción
                $resultado['ESTADO'] = "0";
                $resultado['MENSAJE'] = "Error al insertar el token de sesión en la tabla sesiones: " . mysqli_error($conn);
            }

            // Cerrar la consulta preparada de inserción
            mysqli_stmt_close($stmt_insert_session);
        } else {
            $resultado['ESTADO'] = "0";
            $resultado['MENSAJE'] = "Credenciales incorrectas";
        }
    } else {
        $resultado['ESTADO'] = "0";
        $resultado['MENSAJE'] = "Ingrese datos para el inicio de sesión";
    }
} else {
    // Si la solicitud no es POST, devolver un error
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = "Método no permitido";
}

// Imprimir la respuesta en formato JSON
print_json($resultado);

// Cierre de la conexión a la base de datos
mysqli_close($conn);

function print_json($data)
{
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}
?>
