<?php
error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

try {
    $bodyRequest = file_get_contents("php://input");
    
        // Iniciar transacción
    mysqli_begin_transaction($conn);
    
    $nombre_tabla = 'Empleados';
    $tipo_permiso = 'insertar';
    
    if ($bodyRequest === false) {
        throw new Exception("Error al leer los datos de la solicitud.", 400);
    }

    // Verificar si se enviaron datos en la solicitud
    if (empty($bodyRequest)) {
        throw new Exception("No se han enviado datos en la solicitud.", 400);
    }

    // Decodificar los datos JSON
    $datos = json_decode($bodyRequest, true);
    if ($datos === null) {
        throw new Exception("Error al decodificar los datos JSON.", 400);
    }

    // Verificar si se proporciono el campo Nombre_Sucursal y token
    if (!isset($datos['Nombre_Completo'],$datos['DNI'], $datos['token'])) {
         throw new Exception("Faltan datos requeridos.", 400);
    }

    // Verificar si el campo Nombre_Sucursal y token no está vacío y no consiste solo en espacios en blanco
    if (empty($datos['Nombre_Completo']) ||empty($datos['DNI']) || empty($datos['token']) || trim($datos['token']) === '') {
        throw new Exception("Campos vacíos o con espacios en blanco, llene datos para registro.", 400);
    }


    // Obtener otros datos del array
    $token = $datos['token'];
    $foto = isset($datos['foto']) ? mysqli_real_escape_string($conn, $datos['foto']) : '';
    $Estado_Activo = '1'; //isset($datos['Estado_Activo']) ? mysqli_real_escape_string($conn, $datos['Estado_Activo']) : '';
    $Nombre_Completo = isset($datos['Nombre_Completo']) ? mysqli_real_escape_string($conn, $datos['Nombre_Completo']) : '';
    $Fecha_Nacimiento = isset($datos['Fecha_Nacimiento']) ? mysqli_real_escape_string($conn, $datos['Fecha_Nacimiento']) : '';
    $DNI = isset($datos['DNI']) ? mysqli_real_escape_string($conn, $datos['DNI']) : '';
    $Telefono = isset($datos['Telefono']) ? mysqli_real_escape_string($conn, $datos['Telefono']) : '';
    $Email = isset($datos['Email']) ? mysqli_real_escape_string($conn, $datos['Email']) : '';
    
    $Direccion = isset($datos['Direccion']) ? mysqli_real_escape_string($conn, $datos['Direccion']) : '';
    $Fecha_Inicio = isset($datos['Fecha_Inicio']) ? mysqli_real_escape_string($conn, $datos['Fecha_Inicio']) : '';
    $ID_Sucursal = isset($datos['ID_Sucursal']) ? mysqli_real_escape_string($conn, $datos['ID_Sucursal']) : '';
    $ID_Puesto= isset($datos['ID_Puesto']) ? mysqli_real_escape_string($conn, $datos['ID_Puesto']) : '';
    $Nombre = isset($datos['Nombre']) ? mysqli_real_escape_string($conn, $datos['Nombre']) : '';
    
    $Usuario = isset($datos['Usuario']) ? mysqli_real_escape_string($conn, $datos['Usuario']) : '';
    $contrasena =password_hash($datos['contrasena'], PASSWORD_BCRYPT);
    
     if (empty($datos['Usuario'])) {
        $Usuario = $DNI;
    }
    
    if (empty($datos['contrasena'])) {
        $password = date("dmY", strtotime($Fecha_Nacimiento)); // Formatear la fecha de nacimiento como ddmmyyyy
        $password_hash = password_hash($password, PASSWORD_BCRYPT);
    } else {
        $password_hash = password_hash($datos['contrasena'], PASSWORD_BCRYPT);
    }



    // Verificar autorizacion del usuario y permisos
    $id_usuario = obtenerIdUsuarioPorToken($conn, $token);
    if ($id_usuario === null) {
        throw new Exception("Token de autorizacion invalido.", 401);
    }

    $permiso = tienePermiso($conn, $id_usuario, $nombre_tabla, $tipo_permiso);
    if ($permiso === null) {
        throw new Exception("El usuario no tiene permisos para esta tabla.", 403);
    } elseif (!$permiso) {
        throw new Exception("El usuario no tiene permisos para insertar en esta tabla.", 403);
    }

    // Construir y ejecutar la consulta SQL
    $sql_insert_sucursal = "INSERT INTO Empleados(Nombre_Completo, Fecha_Nacimiento, DNI, Telefono, Email, Direccion, Fecha_Inicio, ID_Sucursal, ID_Puesto, Estado_Activo,Foto) VALUES (?, ?, ?, ?, ?, ?,?,?,?, ?, ?)";
    $stmt = mysqli_prepare($conn, $sql_insert_sucursal);
    mysqli_stmt_bind_param($stmt, "sssssssssss", $Nombre_Completo, $Fecha_Nacimiento, $DNI, $Telefono, $Email, $Direccion, $Fecha_Inicio, $ID_Sucursal,$ID_Puesto,$Estado_Activo,$foto);
    if (!mysqli_stmt_execute($stmt)) {
        throw new Exception("Error al guardar los datos de Empleados: " . mysqli_error($conn), 500);
    }
    // Obtener el ID del empleado insertado
    $empleado_id = mysqli_insert_id($conn);
    
    // Ejecutar la inserción en la tabla usuarios
    $sql_insert_usuario = "INSERT INTO usuarios(ID_Empleado, correo, contrasena) VALUES (?, ?, ?)";
    $stmt = mysqli_prepare($conn, $sql_insert_usuario);
    mysqli_stmt_bind_param($stmt, "iss", $empleado_id, $Usuario, $password_hash);
    if (!mysqli_stmt_execute($stmt)) {
        throw new Exception("Error al guardar los datos de usuario: " . mysqli_error($conn), 500);
    }
    
    // Obtener el ID del usuario insertado
    $id_usuario_insertado = mysqli_insert_id($conn);

        // Insertar datos en la tabla permisos
    $sql_insert_permisos = "
    INSERT INTO permisos (id_usuario, tabla, listar, insertar, eliminar, editar) VALUES
    ($id_usuario_insertado, 'Insumos', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Cabecera_Compras', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Recetas', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Productos', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Produccion', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Cabecera_Ventas', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Sucursales', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Categorias', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Marcas', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Lista_Servicios', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Clientes', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Unidades_Medida', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Empleados', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Puestos_Trabajo', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Sueldos', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Beneficios', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Asistencias', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Ausencias',0, 0, 0, 0),
    ($id_usuario_insertado, 'Adelantos_Sueldo',0, 0, 0, 0),
    ($id_usuario_insertado, 'Turnos', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Proveedores', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Empresa', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Detalle_Ventas', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Metodos_Pago', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Detalle_Compras', 0, 0, 0, 0),
    ($id_usuario_insertado, 'caja', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Cabecera_proforma', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Detalle_proforma', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Cabecera_prestamo', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Detalle_prestamo', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Cabecera_contrato', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Detalle_contrato', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Recetas', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Recetas_Insumos', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Cabecera_produccion', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Produccion', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Cabecera_CreditoVentas', 0, 0, 0, 0),
    ($id_usuario_insertado, 'Cabecera_CreditoCompras', 0, 0, 0, 0),
    ($id_usuario_insertado, 'permisos', 0, 0, 0, 0)";
    
    if (!mysqli_multi_query($conn, $sql_insert_permisos)) {
        throw new Exception("Error al insertar los datos de permisos: " . mysqli_error($conn), 500);
    }
    
    mysqli_stmt_close($stmt);
    
    // Confirmar la transaccion
    mysqli_commit($conn);

    $resultado['ESTADO'] = "1";
    $resultado['MENSAJE'] = "La informacion se registro correctamente :pass: " .$password ." user: ".$Usuario;
    $resultado['CODIGO'] = "200";
 
} catch (Exception $e) {
    // En caso de error, hacer rollback de la transaccion
   // mysqli_rollback($conn);
    
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = $e->getMessage();
    $resultado['CODIGO'] = $e->getCode();
}

// Imprimir respuesta en formato JSON
print_json($resultado);
mysqli_close($conn);

// Funcion para imprimir respuesta en formato JSON
function print_json($data) {
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}

// Funcion para obtener el ID de usuario por token
function obtenerIdUsuarioPorToken($conn, $token) {
    $sql = "SELECT id_usuario FROM sesiones WHERE token = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "s", $token);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $idUsuario);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $idUsuario;
}

// Funcion para verificar si el usuario tiene permisos
function tienePermiso($conn, $idUsuario, $tabla, $tipo_permiso_usuario) {
    $sql = "SELECT $tipo_permiso_usuario FROM permisos WHERE id_usuario = ? AND tabla = ?";
    $stmt = mysqli_prepare($conn, $sql);
    mysqli_stmt_bind_param($stmt, "is", $idUsuario, $tabla);
    mysqli_stmt_execute($stmt);
    mysqli_stmt_bind_result($stmt, $tienePermiso);
    mysqli_stmt_fetch($stmt);
    mysqli_stmt_close($stmt);
    return $tienePermiso;
}
?>
