<?php

error_reporting(E_ALL ^ E_NOTICE);
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, PUT, GET, DELETE, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Content-Type: application/json");

include_once '../conexion.php';

// Obtener datos del cuerpo de la solicitud
$bodyRequest = file_get_contents("php://input");

// Decodificar el cuerpo de la solicitud y guardarlo en un array de PHP
$datos = json_decode($bodyRequest, true);

// Verificar si la solicitud es del tipo POST
if ($_SERVER["REQUEST_METHOD"] === "POST") {
    // Validar datos
    if (!empty($datos['correo']) && !empty($datos['contrasena'])) {
        // Hash de la contraseña
        $password_hash = password_hash($datos['contrasena'], PASSWORD_BCRYPT);

        // Preparar la consulta con consultas preparadas
        $sql = "INSERT INTO usuarios (nombre, correo, contrasena) VALUES (?, ?, ?)";
        $stmt = mysqli_prepare($conn, $sql);

        // Vincular parámetros
        mysqli_stmt_bind_param($stmt, "sss", $datos['nombre'], $datos['correo'], $password_hash);

        // Ejecutar la consulta
        if (mysqli_stmt_execute($stmt)) {
            $resultado['ESTADO'] = "1";
            $resultado['MENSAJE'] = "La información se guardó correctamente";
        } else {
            $resultado['ESTADO'] = "0";
            $resultado['MENSAJE'] = "Error al guardar la información: " . mysqli_stmt_error($stmt);
        }

        // Cerrar la consulta preparada
        mysqli_stmt_close($stmt);
    } else {
        $resultado['ESTADO'] = "0";
        $resultado['MENSAJE'] = "Ingrese datos para el registro";
    }
} else {
    // Si la solicitud no es POST, devolver un error
    $resultado['ESTADO'] = "0";
    $resultado['MENSAJE'] = "Método no permitido";
}

// Imprimir la respuesta en formato JSON
print_json($resultado);

// Cierre de la conexión a la base de datos
mysqli_close($conn);

function print_json($data)
{
    header("HTTP/1.1");
    header("Content-Type: application/json; charset=UTF-8");
    echo json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}
?>
